package com.ruoyi.business.utils;
import java.io.IOException;
import java.io.UnsupportedEncodingException;
import java.nio.charset.Charset;
import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.spec.InvalidKeySpecException;
import java.util.HashMap;

import com.ruoyi.business.domain.HttpDeleteWithBody;
import com.ruoyi.common.exception.ServiceException;
import org.apache.commons.lang3.StringUtils;
import org.apache.http.HttpEntity;
import org.apache.http.HttpResponse;
import org.apache.http.client.HttpClient;
import org.apache.http.client.methods.*;
import org.apache.http.entity.ContentType;
import org.apache.http.entity.StringEntity;
import org.apache.http.impl.client.CloseableHttpClient;
import org.apache.http.impl.client.HttpClients;
import org.apache.http.util.EntityUtils;
import org.slf4j.Logger;
import org.slf4j.LoggerFactory;

public class HttpClientUtil {
	private static final Logger LOGGER = LoggerFactory.getLogger(HttpClientUtil.class);
	private static final HttpClient client = HttpClients.createDefault();


	/**
	 * post请求
	 *
	 * @param post     post
	 * @param paramStr json格式的请求体数据
	 * @return 响应的数据(json)
	 */
	public static String post(HttpPost post, String paramStr, String cookie) {
		String data = "";
		//获取签名请求头
		HashMap<String, String> heads = null;
		try {
			heads = WxSignV3Utils.getSignMap("POST", post.getURI().toString(), paramStr);
		} catch (InvalidKeySpecException e) {
			e.printStackTrace();
		} catch (NoSuchAlgorithmException e) {
			e.printStackTrace();
		} catch (UnsupportedEncodingException e) {
			e.printStackTrace();
		} catch (InvalidKeyException e) {
			e.printStackTrace();
		} catch (SignatureException e) {
			e.printStackTrace();
		}
		post.addHeader("Authorization", heads.get("Authorization"));
		post.addHeader("Accept", heads.get("Accept"));
		post.addHeader("Content-Type",heads.get("Content-Type"));

		if (StringUtils.isNotEmpty(paramStr)) {
			StringEntity entity = new StringEntity(paramStr, ContentType.APPLICATION_JSON);
			post.setEntity(entity);
//			post.setEntity(new StringEntity(paramStr, Charset.forName("UTF-8")));
		}
		try {
			HttpResponse resp = client.execute(post);
			//获取HTTP状态码
			int statusCode = resp.getStatusLine().getStatusCode();
			if (statusCode != 200) {
				System.out.println("Authorization"+heads.get("Authorization"));
				System.out.println("Accept"+heads.get("Accept"));
				System.out.println("Content-Type"+heads.get("Content-Type"));
//				throw new ServiceException("网络错误,状态码:" + statusCode);
			}
			HttpEntity httpEntity = resp.getEntity();
			data = EntityUtils.toString(httpEntity, "UTF-8");
		} catch (Exception e) {
			LOGGER.error("post获取数据异常", e);
		} finally {
			post.releaseConnection();
		}
		return data;
	}


	public static String wxDoPostJson(String url, String json) {
		// 创建Httpclient对象
		CloseableHttpClient httpClient = HttpClients.createDefault();
		CloseableHttpResponse response = null;
		String resultString = "";
		try {
			// 创建Http Post请求
			HttpPost httpPost = new HttpPost(url);

			//获取签名请求头
			HashMap<String, String> heads = null;
			try {
				heads = WxSignV3Utils.getSignMap("POST", url, json);
			} catch (InvalidKeySpecException e) {
				e.printStackTrace();
			} catch (NoSuchAlgorithmException e) {
				e.printStackTrace();
			} catch (UnsupportedEncodingException e) {
				e.printStackTrace();
			} catch (InvalidKeyException e) {
				e.printStackTrace();
			} catch (SignatureException e) {
				e.printStackTrace();
			}

			httpPost.addHeader("Authorization", heads.get("Authorization"));
			httpPost.addHeader("Accept", heads.get("Accept"));
			httpPost.addHeader("Content-Type",heads.get("Content-Type"));
			// 创建请求内容
			StringEntity entity = new StringEntity(json, ContentType.APPLICATION_JSON);
			httpPost.setEntity(entity);
			// 执行http请求
			response = httpClient.execute(httpPost);
//			resultString = EntityUtils.toString(response.getEntity(), getDefaultCharSet());
		} catch (Exception e) {
			e.printStackTrace();
		} finally {
			try {
				response.close();
			} catch (IOException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
		}

		return resultString;
	}

	/**
	 * put请求
	 *
	 * @param put      put
	 * @param paramStr json格式的请求体数据
	 * @return 响应的数据(json)
	 */
	public static String put(HttpPut put, String paramStr, String cookie) {

		String data = "";
		put.addHeader("Content-type", "application/json; charset=utf-8");
		put.setHeader("Accept", "application/json");
		if (StringUtils.isNotEmpty(cookie)) {
			put.setHeader("Cookie", cookie);
		}
		put.setEntity(new StringEntity(paramStr, Charset.forName("UTF-8")));
		try {
			HttpResponse resp = client.execute(put);
			//获取HTTP状态码
			int statusCode = resp.getStatusLine().getStatusCode();
			if (statusCode != 200) {
				throw new ServiceException("网络错误,状态码:" + statusCode);
			}
			HttpEntity httpEntity = resp.getEntity();
			data = EntityUtils.toString(httpEntity, "UTF-8");
		} catch (Exception e) {
			LOGGER.error("put获取数据异常", e);
		} finally {
			put.releaseConnection();
		}
		return data;
	}

	/**
	 * get请求
	 *
	 * @param get get
	 * @return 响应的数据(json)
	 */
	public static String get(HttpGet get, String cookie) {
		String data = "";
		get.addHeader("Content-type", "application/json; charset=utf-8");
		get.setHeader("Accept", "application/json");
		if (StringUtils.isNotEmpty(cookie)) {
			get.setHeader("Cookie", cookie);
		}
		try {
			HttpResponse resp = client.execute(get);
			//获取HTTP状态码
			int statusCode = resp.getStatusLine().getStatusCode();
			if (statusCode != 200) {
				throw new ServiceException("网络错误,状态码:" + statusCode);
			}
			HttpEntity httpEntity = resp.getEntity();
			data = EntityUtils.toString(httpEntity, "UTF-8");
		} catch (Exception e) {
			LOGGER.error("get获取数据异常", e);
		} finally {
			get.releaseConnection();
		}
		return data;
	}

	/**
	 * delete请求
	 *
	 * @param delete delete
	 * @return 响应的数据(json)
	 */
	public static String delete(HttpDelete delete, String cookie) {
		String data = "";
		delete.addHeader("Content-type", "application/json; charset=utf-8");
		delete.setHeader("Accept", "application/json");
		if (StringUtils.isNotEmpty(cookie)) {
			delete.setHeader("Cookie", cookie);
		}
		try {
			HttpResponse resp = client.execute(delete);
			//获取HTTP状态码
			int statusCode = resp.getStatusLine().getStatusCode();
			if (statusCode != 200) {
				throw new ServiceException("网络错误,状态码:" + statusCode);
			}
			HttpEntity httpEntity = resp.getEntity();
			data = EntityUtils.toString(httpEntity, "UTF-8");
		} catch (Exception e) {
			LOGGER.error("get获取数据异常", e);
		} finally {
			delete.releaseConnection();
		}
		return data;
	}

	/**
	 * delete请求
	 *
	 * @param delete
	 * @param paramStr json格式的请求体数据
	 * @return
	 */
	public static String delete(HttpDeleteWithBody delete, String paramStr, String cookie) {
		String data = "";
		delete.addHeader("Content-type", "application/json; charset=utf-8");
		delete.setHeader("Accept", "application/json");
		if (StringUtils.isNotEmpty(cookie)) {
			delete.setHeader("Cookie", cookie);
		}
		delete.setEntity(new StringEntity(paramStr, Charset.forName("UTF-8")));
		try {
			HttpResponse resp = client.execute(delete);
			//获取HTTP状态码
			int statusCode = resp.getStatusLine().getStatusCode();
			if (statusCode != 200) {
				throw new ServiceException("网络错误,状态码:" + statusCode);
			}
			HttpEntity httpEntity = resp.getEntity();
			data = EntityUtils.toString(httpEntity, "UTF-8");
		} catch (Exception e) {
			LOGGER.error("delete获取数据异常", e);
		} finally {
			delete.releaseConnection();
		}
		return data;
	}

}
