package com.ruoyi.business.xhn.service;

import com.alibaba.fastjson.JSONObject;
import com.ruoyi.business.xhn.domain.PayDomain;
import com.ruoyi.business.xhn.domain.RegisteruserUnpaid;
import com.ruoyi.business.xhn.domain.XhnWxVo;
import com.ruoyi.business.xhnnotsingle.registeruser.domain.XhnRegisterUser;

import java.util.List;
import java.util.Map;

/**
 * @author wang_xy
 * @description
 * @Version 1.0
 * @params
 * @return
 * @since 2022/11/21 10:31
 */
public interface WxService {

    /**
     * 微信登录
     * @param code
     * @return
     */
    public XhnRegisterUser wxLogin(String code,String promoter);

    /**
     * 根据微信openid查询会员表中该人员是否授权
     * @param openId
     * @return
     * @throws Exception
     */
    public XhnRegisterUser findUserByOpenId(String openId);

    /**
     * 生成支付订单
     * @param user
     * @return
     * @throws Exception
     */
    public PayDomain getPayPrepayId(XhnRegisterUser user,String description,String total,String purpose,String boxOpenid) throws Exception;
    /**
     * 处理订单付款成功后的业务逻辑
     * @param json
     * @return
     * @throws Exception
     */
    public void payResult(JSONObject json) throws Exception;

    /**
     * 余额换积分接口
     * openid 操作人id
     * mbiId  对应兑换积分id
     * @return
     */
    public void moneyConvertedIntegral(String openid,String mbiId) throws Exception;

    /**
     * 余额开盲盒接口
     * @param openid  消费人openid
     * @param boxOpenid  盲盒人openid
     * @return
     */
    public String moneyOpenBlindBox(String openid,String boxOpenid) throws Exception;
    /**
     * 积分开盲盒接口
     * @param openid  消费人openid
     * @param boxOpenid  盲盒人openid
     * @return
     */
    public String integralOpenBlindBox(String openid,String boxOpenid) throws Exception;

    /**
     * 放入盲盒免费开盲盒一次接口
     * @param openid  消费人openid
     * @param boxOpenid  盲盒人openid
     * @return
     */
    public String freeOpenBlindBox(String openid,String boxOpenid) throws Exception;

    /**
     * 赠送礼物接口
     * @param openid  赠送人openid
     * @param bOpenid  被赠送人openid
     * @param giftId  礼物id
     * @param giftNum  礼物数量
     * @return
     */
    public String giftGive(String openid,String bOpenid,String giftId,Integer giftNum) throws Exception;


    /**
     * 查询本人开盲盒记录
     * @param openid   消费人id（如果只有这个代表查询记录）
     * @param boxOpenid  盲盒人id（如果有这个代表验证是否开过此人的盲盒）
     * @return
     */
    public List<XhnWxVo> getOpenBlindBoxList(String openid, String boxOpenid);

    /**
     * 查询本人都收到过谁的礼物
     * @param openid
     * @return
     */
    public List<XhnWxVo> getReceivedGiftList(String openid);

    /**
     * 查询本人都送过过谁礼物
     * @param openid
     * @return
     */
    public List<XhnWxVo> getGiveGiftList(String openid);

    /**
     * 获取随机一条盲盒信息
     * @param registeruserUnpaid
     * @return
     */
    public RegisteruserUnpaid getRandomBlindBox(RegisteruserUnpaid registeruserUnpaid);

    /**
     * 当然盲盒详情
     * @param openid
     * @return
     */
    public RegisteruserUnpaid findAppUserByOpenid(String openid);

    /**
     * 查询客服信息
     * @return
     */
    public List<Map<String,Object>> getKefuInfo();

}
