package com.ruoyi.business.xhnnotsingle.gift.controller;

import java.util.List;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.xhnnotsingle.gift.domain.XhnGift;
import com.ruoyi.business.xhnnotsingle.gift.service.IXhnGiftService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 礼物信息Controller
 *
 * @author ruoyi
 * @date 2022-11-23
 */
@Controller
@RequestMapping("/xhnnotsingle/gift")
public class XhnGiftController extends BaseController {
    private String prefix = "xhnnotsingle/gift" ;

    @Autowired
    private IXhnGiftService xhnGiftService;

    @RequiresPermissions("xhnnotsingle:gift:view")
    @GetMapping()
    public String gift() {
        return prefix + "/gift" ;
    }

        /**
         * 查询礼物信息列表
         */
        @RequiresPermissions("xhnnotsingle:gift:list")
        @PostMapping("/list")
        @ResponseBody
        public TableDataInfo list(XhnGift xhnGift) {
            startPage();
            List<XhnGift> list = xhnGiftService.selectXhnGiftList(xhnGift);
            return getDataTable(list);
        }

    /**
     * 导出礼物信息列表
     */
    @RequiresPermissions("xhnnotsingle:gift:export")
    @Log(title = "礼物信息" , businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(XhnGift xhnGift) {
        List<XhnGift> list = xhnGiftService.selectXhnGiftList(xhnGift);
        ExcelUtil<XhnGift> util = new ExcelUtil<XhnGift>(XhnGift. class);
        return util.exportExcel(list, "礼物信息数据");
    }

        /**
         * 新增礼物信息
         */
        @GetMapping("/add")
        public String add() {
            return prefix + "/add" ;
        }

    /**
     * 新增保存礼物信息
     */
    @RequiresPermissions("xhnnotsingle:gift:add")
    @Log(title = "礼物信息" , businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(XhnGift xhnGift) {
        return toAjax(xhnGiftService.insertXhnGift(xhnGift));
    }

    /**
     * 修改礼物信息
     */
    @RequiresPermissions("xhnnotsingle:gift:edit")
    @GetMapping("/edit/{id}")
    public String edit(@PathVariable("id") String id, ModelMap mmap) {
        XhnGift xhnGift =
            xhnGiftService.selectXhnGiftById(id);
        mmap.put("xhnGift" , xhnGift);
        return prefix + "/edit" ;
    }

    /**
     * 修改保存礼物信息
     */
    @RequiresPermissions("xhnnotsingle:gift:edit")
    @Log(title = "礼物信息" , businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(XhnGift xhnGift) {
        return toAjax(xhnGiftService.updateXhnGift(xhnGift));
    }

        /**
         * 删除礼物信息
         */
        @RequiresPermissions("xhnnotsingle:gift:remove")
        @Log(title = "礼物信息" , businessType = BusinessType.DELETE)
        @PostMapping("/remove")
        @ResponseBody
        public AjaxResult remove(String ids) {
            return toAjax(xhnGiftService.deleteXhnGiftByIds(ids));
        }
}
