package com.ruoyi.business.xhnnotsingle.order.controller;

import java.util.List;

import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.ResponseBody;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.business.xhnnotsingle.order.domain.XhnOrder;
import com.ruoyi.business.xhnnotsingle.order.service.IXhnOrderService;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.core.page.TableDataInfo;

/**
 * 微信充值订单Controller
 *
 * @author ruoyi
 * @date 2022-11-29
 */
@Controller
@RequestMapping("/xhnnotsingle/order")
public class XhnOrderController extends BaseController {
    private String prefix = "xhnnotsingle/order" ;

    @Autowired
    private IXhnOrderService xhnOrderService;

    @RequiresPermissions("xhnnotsingle:order:view")
    @GetMapping()
    public String order() {
        return prefix + "/order" ;
    }

        /**
         * 查询微信充值订单列表
         */
        @RequiresPermissions("xhnnotsingle:order:list")
        @PostMapping("/list")
        @ResponseBody
        public TableDataInfo list(XhnOrder xhnOrder) {
            startPage();
            List<XhnOrder> list = xhnOrderService.selectXhnOrderList(xhnOrder);
            return getDataTable(list);
        }

    /**
     * 导出微信充值订单列表
     */
    @RequiresPermissions("xhnnotsingle:order:export")
    @Log(title = "微信充值订单" , businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(XhnOrder xhnOrder) {
        List<XhnOrder> list = xhnOrderService.selectXhnOrderList(xhnOrder);
        ExcelUtil<XhnOrder> util = new ExcelUtil<XhnOrder>(XhnOrder. class);
        return util.exportExcel(list, "微信充值订单数据");
    }

        /**
         * 新增微信充值订单
         */
        @GetMapping("/add")
        public String add() {
            return prefix + "/add" ;
        }

    /**
     * 新增保存微信充值订单
     */
    @RequiresPermissions("xhnnotsingle:order:add")
    @Log(title = "微信充值订单" , businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(XhnOrder xhnOrder) {
        return toAjax(xhnOrderService.insertXhnOrder(xhnOrder));
    }

    /**
     * 修改微信充值订单
     */
    @RequiresPermissions("xhnnotsingle:order:edit")
    @GetMapping("/edit/{orderCode}")
    public String edit(@PathVariable("orderCode") String orderCode, ModelMap mmap) {
        XhnOrder xhnOrder =
            xhnOrderService.selectXhnOrderByOrderCode(orderCode);
        mmap.put("xhnOrder" , xhnOrder);
        return prefix + "/edit" ;
    }

    /**
     * 修改保存微信充值订单
     */
    @RequiresPermissions("xhnnotsingle:order:edit")
    @Log(title = "微信充值订单" , businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(XhnOrder xhnOrder) {
        return toAjax(xhnOrderService.updateXhnOrder(xhnOrder));
    }

        /**
         * 删除微信充值订单
         */
        @RequiresPermissions("xhnnotsingle:order:remove")
        @Log(title = "微信充值订单" , businessType = BusinessType.DELETE)
        @PostMapping("/remove")
        @ResponseBody
        public AjaxResult remove(String ids) {
            return toAjax(xhnOrderService.deleteXhnOrderByOrderCodes(ids));
        }
}
