package com.ruoyi.business.xhnnotsingle.registeruser.controller;

import com.ruoyi.business.xhnnotsingle.registeruser.domain.XhnRegisterUser;
import com.ruoyi.business.xhnnotsingle.registeruser.service.IXhnRegisterUserService;
import com.ruoyi.common.annotation.Log;
import com.ruoyi.common.core.controller.BaseController;
import com.ruoyi.common.core.domain.AjaxResult;
import com.ruoyi.common.core.page.TableDataInfo;
import com.ruoyi.common.enums.BusinessType;
import com.ruoyi.common.utils.DateUtils;
import com.ruoyi.common.utils.poi.ExcelUtil;
import com.ruoyi.common.utils.uuid.IdUtils;
import org.apache.shiro.authz.annotation.RequiresPermissions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Controller;
import org.springframework.ui.ModelMap;
import org.springframework.web.bind.annotation.*;

import java.util.List;

/**
 * 小红娘会员注册信息Controller
 *
 * @author ruoyi
 * @date 2022-11-23
 */
@Controller
@RequestMapping("/xhnnotsingle/registeruser")
public class XhnRegisterUserController extends BaseController {
    private String prefix = "xhnnotsingle/registeruser";

    @Autowired
    private IXhnRegisterUserService xhnRegisterUserService;

    @RequiresPermissions("xhnnotsingle:registeruser:view")
    @GetMapping()
    public String registeruser() {
        return prefix + "/registeruser";
    }

    /**
     * 查询小红娘会员注册信息列表
     */
    @RequiresPermissions("xhnnotsingle:registeruser:list")
    @PostMapping("/list")
    @ResponseBody
    public TableDataInfo list(XhnRegisterUser xhnRegisterUser) {
        startPage();
        List<XhnRegisterUser> list = xhnRegisterUserService.selectXhnRegisterUserList(xhnRegisterUser);
        return getDataTable(list);
    }

    /**
     * 导出小红娘会员注册信息列表
     */
    @RequiresPermissions("xhnnotsingle:registeruser:export")
    @Log(title = "小红娘会员注册信息", businessType = BusinessType.EXPORT)
    @PostMapping("/export")
    @ResponseBody
    public AjaxResult export(XhnRegisterUser xhnRegisterUser) {
        List<XhnRegisterUser> list = xhnRegisterUserService.selectXhnRegisterUserList(xhnRegisterUser);
        ExcelUtil<XhnRegisterUser> util = new ExcelUtil<XhnRegisterUser>(XhnRegisterUser.class);
        return util.exportExcel(list, "小红娘会员注册信息数据");
    }

    /**
     * 新增小红娘会员注册信息
     */
    @GetMapping("/add")
    public String add() {
        return prefix + "/add";
    }

    /**
     * 新增保存小红娘会员注册信息
     */
    @RequiresPermissions("xhnnotsingle:registeruser:add")
    @Log(title = "小红娘会员注册信息", businessType = BusinessType.INSERT)
    @PostMapping("/add")
    @ResponseBody
    public AjaxResult addSave(XhnRegisterUser xhnRegisterUser) {
        xhnRegisterUser.setOpenid(IdUtils.fastUUID());
        xhnRegisterUser.setDelisting(String.valueOf(0));
        xhnRegisterUser.setIsLogin(String.valueOf(3));
        return toAjax(xhnRegisterUserService.insertXhnRegisterUser(xhnRegisterUser));
    }

    /**
     * 修改小红娘会员注册信息
     */
    @RequiresPermissions("xhnnotsingle:registeruser:edit")
    @GetMapping("/edit/{openid}")
    public String edit(@PathVariable("openid") String openid, ModelMap mmap) {
        XhnRegisterUser xhnRegisterUser =
                xhnRegisterUserService.selectXhnRegisterUserByOpenid(openid);
        mmap.put("xhnRegisterUser", xhnRegisterUser);
        return prefix + "/edit";
    }

    /**
     * 修改保存小红娘会员注册信息
     */
    @RequiresPermissions("xhnnotsingle:registeruser:edit")
    @Log(title = "小红娘会员注册信息", businessType = BusinessType.UPDATE)
    @PostMapping("/edit")
    @ResponseBody
    public AjaxResult editSave(XhnRegisterUser xhnRegisterUser) {
        return toAjax(xhnRegisterUserService.updateXhnRegisterUser(xhnRegisterUser));
    }

    /**
     * 删除小红娘会员注册信息
     */
    @RequiresPermissions("xhnnotsingle:registeruser:remove")
    @Log(title = "小红娘会员注册信息", businessType = BusinessType.DELETE)
    @PostMapping("/remove")
    @ResponseBody
    public AjaxResult remove(String ids) {
        return toAjax(xhnRegisterUserService.deleteXhnRegisterUserByOpenids(ids));
    }

    /**
     * 脱单状态
     */
    @PostMapping("/tuodan")
    @ResponseBody
    public AjaxResult tuodan(String openIds) {
        XhnRegisterUser xhnRegisterUser = new XhnRegisterUser();
        xhnRegisterUser.setOpenid(openIds);
//        xhnRegisterUser.setUpdateBy(ShiroUtils.getUserId());
        xhnRegisterUser.setUpdateTime(DateUtils.getNowDate());
//        xhnRegisterUser.setup(ShiroUtils.getSysUser().getUserName());
        return toAjax(xhnRegisterUserService.updateisLoginById(xhnRegisterUser));
    }
}
